@@ -1,13 +1,14 @@
 # simulate.table : Generates simulated contingency tables (with or without noise) of a given "type".
 #
 # Parameters details
-# n              : Expected sample size, should be atleast nrow for "functional" and "nonmonotonic",
-#                  nrow + 1 for "dependent.non.functional" and nrow * ncol for "independent" [DEFAULT = 100].
-# nrow           : Expected number of rows in the table, should be greater than 2 (greater than 3 for type = "nonmonotonic") [DEFAULT = 3].
+# n              : Expected sample size, should be atleast nrow for "functional", "many.to.one", "discontinuous"
+#                  and nrow * ncol for "independent" and "dependent.non.functional" [DEFAULT = 100].
+# nrow           : Expected number of rows in the table, should be greater than 2 (greater than 3 for type = "many.to.one") [DEFAULT = 3].
 # ncol           : Expected number of columns in the table , should be greater than 2 [DEFAULT = 3].
 # types include  : functional: y is a function of x but x may or may not be a function of y [DEFAULT] ,
-#                  nonmonotonic: y is a function of x and x is not a function of y,
+#                  many.to.one: y is a function of x and x is not a function of y,
 #                  dependent.non.functional: Non functional table with statistical dependency,
+#                : discontinuous: y is a duscontinuous function of x but x may or may not be function of y,
 #                  independent: Independent tables (null population).
 # n.tables       : number of tables to be generated.[DEFAULT = 1].
 # row.marginal   : A vector of row probabilities [DEFAULT = Equally likely].
@@ -16,16 +17,17 @@
 #
 # Created by     : Ruby Sharma
 # Date           : October 16 2016
-# Last Modified  : February 25 2017
-# Version        : 0.0.0
+# Last Modified  : April 20 2017
+# Version        : 0.0.1
+
 
-# source("house_noise_model.R")
 
 simulate_tables <- function(n=100, nrow=3, ncol=3,
                             type = c("functional",  #=F
-                                    "nonmonotonic", #=NI
+                                    "many.to.one", #= MO
                                     "independent", #=I
-                                    "dependent.non.functional" #=NF
+                                    "dependent.non.functional", #=NF
+                                    "discontinuous" #=DS
                                     ), noise=0.0, n.tables=1,
                             row.marginal=rep(1/nrow, nrow),
                             col.marginal= rep(1/ncol, ncol))
@@ -64,6 +66,7 @@ simulate_tables <- function(n=100, nrow=3, ncol=3,
 
 
   #return a list of pattern table, sampled contingency table and noise table.
+
   list(pattern.list = pattern.list, sample.list = sample.list, noise.list = noise.list, pvalue.list = p.value.list)
 }
 
@@ -74,29 +77,29 @@ table.generate=function(nrow, ncol, type, n, noise, row.marginal, col.marginal)
 
   if(type=="dependent.non.functional"){
 
-    if(n<(nrow+1))
-      stop(paste("For dependent.non.functional, n should be greater than or equal to",(nrow+1)))
+    if(n<(nrow*ncol))
+      stop(paste("For dependent.non.functional, n should be greater than or equal to",(nrow*ncol)))
 
     sam.val.row = sample.in.rows(n, row.marginal, type, ncol)
     pattern.table = nonfunctional.table(nrow, ncol, row.marginal, sam.val.row)
     prob.table = non.functional.prob(nrow, ncol, pattern.table)
     sampled.table = dis.sample.prob(nrow, ncol, sam.val.row, prob.table)
+    sampled.table = is_dependent(n, nrow, ncol, row.marginal, col.marginal, sampled.table, sam.val.row)
     p.val = chisq.test.pval(sampled.table)
 
-
-  } else if (type=="nonmonotonic") {
+  } else if (type=="many.to.one") {
 
     if(nrow<3)
-      stop("For nonmonotonic, number of rows should atleast be 3")
+      stop("For many.to.one, number of rows should be at least be 3!")
 
     if(length(which(row.marginal!=0))<3)
-      stop("For nonmonotonic, atleast three non-zero row probabilites expected")
+      stop("For many.to.one, at least three non-zero row probabilities are expected!")
 
     if(n<nrow)
-      stop(paste("n should be greater than or equal to",nrow))
+      stop(paste("n should be greater than or equal to", nrow))
 
     sam.val.row = sample.in.rows(n, row.marginal, type, ncol)
-    pattern.table = nonmonotonic.table(nrow, ncol, row.marginal, sam.val.row)
+    pattern.table = many.to.one.table(nrow, ncol, row.marginal, sam.val.row)
     sampled.table = dis.sample.prob(nrow, ncol, sam.val.row, pattern.table)
     p.val=FunChisq::fun.chisq.test(sampled.table)$p.value
 
@@ -114,7 +117,16 @@ table.generate=function(nrow, ncol, type, n, noise, row.marginal, col.marginal)
     sampled.table = dis.sample.prob(nrow, ncol, sam.val.row, prob.table)
     p.val = chisq.test.pval(sampled.table)
 
-  } else {
+  } else if(type=="discontinuous"){
+    if(n<nrow)
+      stop(paste("n should be greater than or equal to",nrow))
+
+    sam.val.row = sample.in.rows(n, row.marginal, type, ncol)
+    pattern.table = discontinuous.table(nrow, ncol, row.marginal, sam.val.row)
+    sampled.table = dis.sample.prob(nrow, ncol, sam.val.row, pattern.table)
+    p.val=FunChisq::fun.chisq.test(sampled.table)$p.value
+
+  }else {
 
     if(n<nrow)
       stop(paste("n should be greater than or equal to",nrow))
@@ -129,7 +141,7 @@ table.generate=function(nrow, ncol, type, n, noise, row.marginal, col.marginal)
   if(noise==0){
     noise.table = sampled.table
   } else {
-    noise.table = add.house.noise(tables = sampled.table, u = noise, margin = 1)
+    noise.table = add.house.noise(tables = sampled.table, u = noise, margin = 2)
   }
 
 
@@ -140,7 +152,6 @@ table.generate=function(nrow, ncol, type, n, noise, row.marginal, col.marginal)
 # distributing samples across rows guided by row probabilities
 sample.in.rows=function(n, row.marginal, type, ncol)
 {
-
   non.zero.rows.length = length(which(row.marginal!=0))
   if(type!="independent") {
 
@@ -157,8 +168,6 @@ sample.in.rows=function(n, row.marginal, type, ncol)
     ind = which(row.marginal!=0)
     sam.val[ind] = sam.val[ind]+ncol
 
-
-
   }
 
   return(sam.val)
@@ -168,9 +177,7 @@ sample.in.rows=function(n, row.marginal, type, ncol)
 # distribute samples
 dis.sample.prob=function(nrow, ncol, sam.val.row, table)
 {
-
-  sampled.table=as.data.frame(matrix(0,nrow = nrow,ncol = ncol))
-
+  sampled.table=as.data.frame(matrix(0, nrow = nrow, ncol = ncol))
   for(i in 1:nrow)
   {
     # determine if all columns in the ith row of the supplied table are zero
@@ -192,7 +199,6 @@ dis.sample.prob=function(nrow, ncol, sam.val.row, table)
 # generating pattern table for "functional"
 functional.table=function(nrow, ncol, row.marginal, sam.val.row)
 {
-
   pattern.table = as.data.frame(matrix(0,ncol = ncol,nrow = nrow))
 
   for(i in 1:nrow)
@@ -212,15 +218,15 @@ functional.table=function(nrow, ncol, row.marginal, sam.val.row)
 }
 
 
-# generating pattern table for "nonmonotonic"
-nonmonotonic.table=function(nrow, ncol, row.marginal, sam.val.row)
+# generating pattern table for "many.to.one"
+many.to.one.table=function(nrow, ncol, row.marginal, sam.val.row)
 {
 
   pattern.table = as.data.frame(matrix(0,ncol = ncol,nrow = nrow))
   # get the functional table
   pattern.table = functional.table(nrow, ncol, row.marginal, sam.val.row)
   # check for non-monotonicity
-  pattern.table = is_nonmonotonic(pattern.table)
+  pattern.table = is_many.to.one(pattern.table)
 
   return(pattern.table)
 }
@@ -262,32 +268,114 @@ nonfunctional.table=function(nrow, ncol, row.marginal, sam.val.row)
       pattern.table[i,index] = 1
     }
   }
-
-  pattern.table = make.non.functional(pattern.table, ncol, sam.val.row)
+  pattern.table = make.non.functional(pattern.table, ncol, sam.val.row, nonfunc = "notf.x")
+  pattern.table = make.non.functional(t(pattern.table), ncol, sam.val.row, nonfunc = "notf.y")
+  pattern.table = t(pattern.table)
   return(pattern.table)
 }
 
+# generating pattern table for "discontinuous function"
+discontinuous.table=function(nrow, ncol, row.marginal, sam.val.row)
+{
+  pattern.table = as.data.frame(matrix(0,ncol = ncol,nrow = nrow))
+
+  sample.from =seq_along(1:ncol(pattern.table))
+  prev.col.ind = NULL
+
+  for(i in 1:nrow)
+  {
+    if(sam.val.row[i]!=0) {
+
+      index = sample(sample.from,1)
+      sample.from =seq_along(1:ncol(pattern.table))
+      prev.col.ind = index
+      pattern.table[i,index] = 1
+      sample.from = sample.from[-which(sample.from==prev.col.ind)]
+
+
+    }
+  }
+  # check for constant functions
+  pattern.table=not_constant(ncol, pattern.table)
+  return(pattern.table)
+}
 
 # introducting atleast two entries in one row for nonfunctional table
-make.non.functional=function(pattern.table, ncol, sam.val.row)
+make.non.functional=function(pattern.table, ncol, sam.val.row, nonfunc)
 {
   indexes = non.zero.index(pattern.table)
   rows = indexes$rows
   cols = indexes$cols
 
-  if(anyDuplicated(rows)==0) {
 
-    only.row = which(sam.val.row>1)
-    chng.row.index = sample(rows[only.row],1)
+  if(nonfunc == "notf.x")
+  {
+    if(anyDuplicated(rows)==0) {
+      only.row = which(sam.val.row>1)
+      chng.from = c(only.row)
+      if(length(chng.from)==1)
+      {
+        chng.row.index = chng.from
+      }else
+      {
+        chng.row.index = sample(chng.from, 1)
+      }
 
-    if(length(rows[only.row])==1)
-      chng.row.index = rows[only.row]
+      prev.col.index = cols[which(rows==chng.row.index)]
+      chng.col.index = sample.sec.col.ind(prev.col.index,ncol)
+      pattern.table[chng.row.index,chng.col.index] = 1
+    }
 
-    prev.col.index = cols[which(rows==chng.row.index)]
-    chng.col.index = sample.sec.col.ind(prev.col.index,ncol)
-    pattern.table[chng.row.index,chng.col.index] = 1
   }
 
+  if(nonfunc == "notf.y")
+  {
+    nr = nrow(pattern.table)
+    nc = ncol(pattern.table)
+    wth.more.samp <- c()
+    wth.less.samp <- c()
+    for(i in 1:nc)
+    {
+      sample.lim <- length(cols[cols[]==i])
+      if(sam.val.row[i]>sample.lim)
+      {
+        wth.more.samp = c(i, wth.more.samp)
+      }else{
+        wth.less.samp = c(i, wth.less.samp )
+      }
+
+    }
+    if(anyDuplicated(rows)==0) {
+
+
+      chng.from = rows
+      if(length(chng.from)==1)
+      {
+        chng.row.index = chng.from
+      }else
+      {
+        chng.row.index = sample(chng.from, 1)
+      }
+
+      prev.col.index = cols[which(rows==chng.row.index)]
+      chng.col.index = sample.sec.col.ind_notf.y(prev.col.index, nc, wth.more.samp)
+      if(chng.col.index == 0)
+      {
+
+        if(length(rows[cols[]==wth.less.samp])==1)
+        {
+          chng.row.index = rows[cols[]==wth.less.samp]
+        }else{
+          chng.row.index = sample(rows[cols[]==wth.less.samp], 1)
+        }
+        chng.col.index = wth.more.samp
+      }
+      pattern.table[chng.row.index,chng.col.index] = 1
+    }
+
+  }
+
+
   return(pattern.table)
 }
 
@@ -354,7 +442,7 @@ not_constant=function(ncol, pattern.table)
 
 
 # check monotonicity, if found then atleast two rows would share samples in the same column
-is_nonmonotonic=function(pattern.table)
+is_many.to.one=function(pattern.table)
 {
 
   indexes = non.zero.index(pattern.table)
@@ -370,6 +458,41 @@ is_nonmonotonic=function(pattern.table)
 
   return(pattern.table)
 }
+#check whether the dependent.non.functional table is dependent, if not make dependent
+is_dependent = function(n, nrow, ncol, row.marginal,  col.marginal, sampled.table, sam.val.row)
+{
+
+  sam.val.indep =  sample.in.rows(n, row.marginal, type ="independent", ncol)
+  expec.prob.table = as.data.frame(matrix(0,ncol = ncol,nrow = nrow))
+  expec.prob.table = indep.prob.table(nrow, ncol, row.marginal, col.marginal)
+  indep.sampled.table = dis.sample.prob(nrow, ncol, sam.val.indep, expec.prob.table)
+  difference = indep.sampled.table - sampled.table
+  if(all(difference==0))
+  {
+    indexes = non.zero.index(sampled.table)
+    rows = indexes$rows
+    cols = indexes$cols
+    sel.row <- sample(rows, 1)
+    sel.from <- cols[which(rows==sel.row)]
+    if(length(sel.from)==1)
+    {
+      sel.colj1 = sel.from
+    }else
+    {
+      sel.colj1 <- sample(sel.from,1)
+    }
+    sel.from <- 1:ncol
+    sel.colj2 <-sample(sel.from[-which(sel.from==sel.colj1)],1)
+    sampled.table[sel.row,sel.colj2] <- sampled.table[sel.row,sel.colj2] + sampled.table[sel.row,sel.colj1]
+    sampled.table[sel.row,sel.colj1] <- 0
+  }
+
+  return(sampled.table)
+
+
+
+}
+
 
 
 # sorting row and column indexes on the basis of row
@@ -412,10 +535,13 @@ sample.sec.col.ind=function(index, ncol)
   if(index>1 && index<ncol)
     vec.to.sample = c(1:(index-1),(index+1):ncol)
 
-  chng.col.index = sample(vec.to.sample,1)
-
   if(length(vec.to.sample)==1)
+  {
     chng.col.index=vec.to.sample
+  }else{
+    chng.col.index = sample(vec.to.sample,1)
+  }
+
 
   return(chng.col.index)
 }
@@ -431,8 +557,9 @@ prelim.check = function(nrow, ncol, type, n, noise, row.marginal, col.marginal)
   if(class(type)!="character")
     stop("type should be character")
 
-  if(type!="functional" && type!="nonmonotonic" && type!="dependent.non.functional" && type!="independent")
-    stop("type can only be functional, nonmonotonic, dependent.non.functional or independent")
+  if(type!="functional" && type!="many.to.one" && type!="dependent.non.functional" &&
+     type!="independent"&& type!="discontinuous")
+    stop("type can only be functional, many.to.one, dependent.non.functional, independent or discontinuous")
 
   if(class(n)!="numeric" && class(n)!="integer")
     stop("n should be numeric")
@@ -455,7 +582,7 @@ chisq.test.pval <- function(table)
   non.zero.cols <- apply(table, 2, function(col) { 0 != sum(abs(col)) } )
 
   # perform Pearson chi-square test
-  chisq <- chisq.test(table[non.zero.rows, non.zero.cols])$statistic
+  chisq <- suppressWarnings(chisq.test(table[non.zero.rows, non.zero.cols])$statistic)
 
   # compute p-value using the orgional table size
   pval <- pchisq( chisq, prod(dim(table) - 1), lower.tail = FALSE)
@@ -463,3 +590,35 @@ chisq.test.pval <- function(table)
   return(pval)
 }
 
+#sampling second column for making x!=f(y) for dependent.non.functional
+sample.sec.col.ind_notf.y <- function(index ,ncol, only.col)
+{
+
+
+  if(index==1)
+    vec.to.sample = c(2:ncol)
+
+  if(index==ncol)
+    vec.to.sample = c(1:(ncol-1))
+
+  if(index>1 && index<ncol)
+    vec.to.sample = c(1:(index-1),(index+1):ncol)
+
+  vec.to.sample =  vec.to.sample[vec.to.sample %in% only.col]
+
+  if(length(vec.to.sample)==0)
+  {
+    chng.col.index = 0
+  }else if(length(vec.to.sample)==1)
+  {
+    chng.col.index = vec.to.sample
+  }else{
+    chng.col.index = sample(vec.to.sample,1)
+  }
+   return(chng.col.index)
+
+
+
+}
+
+
